!-------------------------------------------------------------------------------

!                      Code_Saturne version 3.0.0
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2013 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_modules.f90
!>
!> \brief User-defined module: it allows to create any user array.
!>
!> This file is compiled before all other user Fortran files.
!> To ensure this, it must not be renamed.
!>
!> The user may define an arbitrary number of modules here, even though
!> only one is defined in the example.
!
!-------------------------------------------------------------------------------

module user_module

  ! Example: allocatable user arrays

  double precision,save::sumavdt,ckp
  integer, save :: nymax
  integer, ALLOCATABLE,save,DIMENSION(:)   :: corrs,invarc
  double precision, ALLOCATABLE,save,DIMENSION(:) ::ydat
  !--SGS Model
  integer::SGSMODEL
  parameter(SGSMODEL=0)
  !--Explicit algebraic model

  double precision, ALLOCATABLE,save,DIMENSION(:)   :: bet1,bet4
  double precision, ALLOCATABLE,save,DIMENSION(:)   :: ksgs
  double precision, ALLOCATABLE,save,DIMENSION(:)   :: cksgs
  double precision, ALLOCATABLE,save,DIMENSION(:)   :: Diss
  double precision, ALLOCATABLE,save,DIMENSION(:,:) :: Fsgs
  double precision, ALLOCATABLE,save,DIMENSION(:,:) :: tauij
  double precision, ALLOCATABLE,save,DIMENSION(:,:) :: tauijnl
  double precision, ALLOCATABLE,save,DIMENSION(:,:) :: Sij
  double precision, ALLOCATABLE,save,DIMENSION(:)   :: Sabs
  double precision, ALLOCATABLE,save,DIMENSION(:)   :: ts
  double precision, ALLOCATABLE,save,DIMENSION(:)   :: cp1
  double precision, ALLOCATABLE,save,DIMENSION(:)   :: volumef
!--stochastic variable
  double precision, ALLOCATABLE,save,DIMENSION(:)   :: gstor
!--Statistical averages
  double precision, ALLOCATABLE, save,DIMENSION(:)  ::    ksgsav
  double precision, ALLOCATABLE, save,DIMENSION(:)  ::    Dissav,bscatav
  double precision, ALLOCATABLE, save,DIMENSION(:)  ::     nutav
  double precision, ALLOCATABLE, save,DIMENSION(:)  ::   cksgsav
  double precision, ALLOCATABLE, save,DIMENSION(:,:)::    Fsgsav
  double precision, ALLOCATABLE, save,DIMENSION(:,:)::   tauijav
  double precision, ALLOCATABLE, save,DIMENSION(:,:):: tauijnlav
  double precision, ALLOCATABLE, save,DIMENSION(:)  ::    Sabsav
  double precision, ALLOCATABLE, save,DIMENSION(:,:)::     Sijav
  double precision, ALLOCATABLE, save,DIMENSION(:)::    bet1av,bet4av
  double precision, ALLOCATABLE, save,DIMENSION(:)::      tsav
  double precision, ALLOCATABLE, save,DIMENSION(:)::     cp1av

contains

  !=============================================================================

  ! Allocate arrays

  subroutine init_user_module(ncel,ncelet,SGSMODEL)

    ! Arguments

    integer, intent(in) :: ncel,ncelet,SGSMODEL

    ! Local variables

    integer :: err = 0

    if (err .eq. 0 .and. .not.allocated(corrs)) then
       allocate(corrs(ncel), stat=err)
    endif
    if (err .eq. 0 .and. .not.allocated(invarc)) then
       allocate(invarc(nymax), stat=err)
    endif

    if (err .eq. 0 .and. .not.allocated(ydat)) then
       allocate(ydat(nymax), stat=err)
    endif
    if (SGSMODEL.eq.2) then
       ALLOCATE(   ksgsav(nymax))
       ALLOCATE(   Dissav(nymax))
       ALLOCATE(    nutav(nymax))
       ALLOCATE(  cksgsav(nymax))
       ALLOCATE(   Fsgsav(nymax,3))
       ALLOCATE(  tauijav(nymax,6))
       ALLOCATE(tauijnlav(nymax,6))
       ALLOCATE(   Sabsav(nymax))
       ALLOCATE(    Sijav(nymax,6))
       ALLOCATE(   bet1av(nymax),bet4av(nymax))
       ALLOCATE(     tsav(nymax))
       ALLOCATE(    cp1av(nymax))
       !--initialize averages
       ksgsav    = 0.d0
       Dissav    = 0.d0
       nutav     = 0.d0
       cksgsav   = 0.d0
       bet1av    = 0.d0
       bet4av    = 0.d0
       tsav      = 0.d0
       Fsgsav    = 0.d0
       tauijav   = 0.d0
       tauijnlav = 0.d0
       Sabsav    = 0.d0
       Sijav     = 0.d0
       cp1av     = 0.d0
       sumavdt   = 0.d0
       write(nfecra,*) 'Averages are reset'

    endif

    if (err /= 0) then
      write (*, *) "Error allocating array."
      call csexit(err)
    endif

    return

  end subroutine init_user_module

  !=============================================================================

  ! Free related arrays

  subroutine finalize_user_module

    if (allocated(corrs)) then
       deallocate(corrs)
    endif
    if (allocated(invarc)) then
       deallocate(invarc)
    endif

    if (allocated(ydat)) then
       deallocate(ydat)
    endif
    
  end subroutine finalize_user_module

  !=============================================================================

end module user_module
